package org.obm.sync.calendar;

import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import org.obm.sync.base.Category;
import org.obm.sync.items.AbstractItemsParser;
import org.obm.sync.items.EventChanges;
import org.obm.sync.utils.DOMUtils;
import org.obm.sync.utils.DateHelper;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

public class CalendarItemsParser extends AbstractItemsParser {

	public EventChanges parseChanges(Document doc) {
		EventChanges changes = new EventChanges();
		Element root = doc.getDocumentElement();
		Date lastSync = DateHelper.asDate(root.getAttribute("lastSync"));
		changes.setLastSync(lastSync);
		Element removed = DOMUtils.getUniqueElement(root, "removed");
		Element updated = DOMUtils.getUniqueElement(root, "updated");

		NodeList rmed = removed.getElementsByTagName("event");
		List<String> removedIds = new ArrayList<String>(rmed.getLength() + 1);
		for (int i = 0; i < rmed.getLength(); i++) {
			Element e = (Element) rmed.item(i);
			removedIds.add(e.getAttribute("id"));
		}
		changes.setRemoved(removedIds.toArray(new String[removedIds.size()]));

		NodeList upd = updated.getElementsByTagName("event");
		List<Event> updatedEvents = new ArrayList<Event>(upd.getLength() + 1);
		for (int i = 0; i < upd.getLength(); i++) {
			Element e = (Element) upd.item(i);
			Event ev = parseEvent(e);
			updatedEvents.add(ev);
		}
		changes.setUpdated(updatedEvents
				.toArray(new Event[updatedEvents.size()]));

		return changes;
	}

	public Event parseEvent(Element e) {
		Event ev = new Event();
		ev.setUid(e.getAttribute("id"));
		ev.setAllday(e.hasAttribute("allDay") ? "true".equals(e
				.getAttribute("allDay")) : false);
		ev.setType(EventType.valueOf(e.getAttribute("type")));
		ev.setExtId(s(e, "extId"));
		
		ev.setParentId(i(e, "parentId"));
		
		String opacity = DOMUtils.getElementTextInChildren(e, "opacity");
		if (opacity != null) {
			ev.setOpacity(EventOpacity.valueOf(opacity));
		}
		ev.setTitle(s(e, "title"));
		ev.setOwner(s(e, "owner"));
		ev.setOwnerEmail(s(e, "ownerEmail"));
		ev.setDescription(s(e, "description"));
		ev.setDate(d(e, "date"));
		ev.setPrivacy(i(e, "privacy"));
		ev.setPriority(i(e, "priority"));
		ev.setDuration(i(e, "duration"));
		ev.setCategory(s(e, "category"));
		ev.setLocation(s(e, "location"));
		ev.setAlert(i(e, "alert"));
		ev.setTimeUpdate(d(e, "timeupdate"));

		Element ats = DOMUtils.getUniqueElement(e, "attendees");
		parseAttendees(ev, ats);

		Element rec = DOMUtils.getUniqueElement(e, "recurrence");
		if (rec != null) {
			parseRecurrence(ev, rec);
		}
		
		Element eventsExp = DOMUtils.getUniqueElement(e, "eventExceptions");
		if (eventsExp != null) {
			NodeList elems = eventsExp.getElementsByTagName("eventException");
			for(int i = 0; i<elems.getLength();i++){
				Event eexcept = parseEvent((Element)elems.item(i));
				ev.getRecurrence().addEventException(eexcept);
			}
			
		}
		return ev;
	}

	private void parseAttendees(Event ev, Element ats) {
		String[][] atVals = DOMUtils.getAttributes(ats, "attendee",
				new String[] { "displayName", "email", "state", "required", "percent"});
		List<Attendee> la = new ArrayList<Attendee>(atVals.length);
		for (String[] attendee : atVals) {
			Attendee at = new Attendee();
			at.setDisplayName(attendee[0]);
			at.setEmail(attendee[1]);
			at.setState(ParticipationState.getValueOf(attendee[2]));
			at.setRequired(ParticipationRole.valueOf(attendee[3]));
			if (attendee[4] != null && !attendee[4].equals("")) {
				at.setPercent(Integer.parseInt(attendee[4]));
			} else {
				at.setPercent(100);
			}
			
			la.add(at);
		}
		ev.setAttendees(la);
	}

	private void parseRecurrence(Event ev, Element rec) {
		String kind = rec.getAttribute("kind");
		EventRecurrence er = new EventRecurrence();
		er.setKind(RecurrenceKind.valueOf(kind));
		if (!"none".equals(kind)) {
			if (rec.hasAttribute("end")) {
				er.setEnd(DateHelper.asDate(rec.getAttribute("end")));
			}
			er.setFrequence(Integer.parseInt(rec.getAttribute("freq")));
			if (RecurrenceKind.weekly == er.getKind()) {
				er.setDays(rec.getAttribute("days"));
			}
			String[] exDates = DOMUtils.getTexts(rec, "exception");
			for (int i = 0; i < exDates.length; i++) {
				er.addException(DateHelper.asDate(exDates[i]));
				
			}
		} else {
			er.setEnd(new Date());
		}
		ev.setRecurrence(er);

	}

	public CalendarInfo[] parseInfos(Document doc) {
		NodeList infosList = doc.getElementsByTagName("info");
		CalendarInfo[] infos = new CalendarInfo[infosList.getLength()];
		for (int i = 0; i < infosList.getLength(); i++) {
			infos[i] = parseInfo((Element) infosList.item(i));
		}
		return infos;
	}

	private CalendarInfo parseInfo(Element item) {
		CalendarInfo ci = new CalendarInfo();
		ci.setFirstname(s(item, "first"));
		ci.setLastname(s(item, "last"));
		ci.setMail(s(item, "mail"));
		ci.setUid(s(item, "uid"));
		ci.setRead("true".equals(s(item, "read")));
		ci.setWrite("true".equals(s(item, "write")));
		return ci;
	}

	public List<Category> parseCategories(Element documentElement) {
		NodeList nl = documentElement.getElementsByTagName("cat");
		ArrayList<Category> ret = new ArrayList<Category>(nl.getLength());
		for (int i = 0; i < nl.getLength(); i++) {
			Element cat = (Element) nl.item(i);
			Category c = new Category();
			c.setId(Integer.parseInt(cat.getAttribute("id")));
			c.setLabel(cat.getAttribute("label"));
			ret.add(c);
		}
		return ret;
	}

	public List<Event> parseListEvents(Element documentElement) {
		List<Event> ret = new LinkedList<Event>();
		NodeList nl = documentElement.getElementsByTagName("event");
		for (int i = 0; i < nl.getLength(); i++) {
			Element e = (Element) nl.item(i);
			ret.add(parseEvent(e));
		}
		return ret;
	}

	public List<EventTimeUpdate> parseListEventTimeUpdate(
			Element documentElement) {
		List<EventTimeUpdate> ret = new LinkedList<EventTimeUpdate>();
		NodeList nl = documentElement.getElementsByTagName("eventTimeUpdate");
		for (int i = 0; i < nl.getLength(); i++) {
			Element e = (Element) nl.item(i);
			ret.add(parseEventTimeUpdate(e));
		}
		return ret;
	}

	private EventTimeUpdate parseEventTimeUpdate(Element e) {
		EventTimeUpdate ev = new EventTimeUpdate();
		ev.setUid(e.getAttribute("id"));
		ev.setExtId(s(e, "extId"));
		ev.setTimeUpdate(d(e, "timeupdate"));
		ev.setParentId(Integer.parseInt(s(e, "parentId")));
		return ev;
	}
}
