package org.obm.sync.calendar;

import java.io.ByteArrayOutputStream;
import java.util.Date;
import java.util.List;

import org.obm.sync.base.Category;
import org.obm.sync.items.AbstractItemsWriter;
import org.obm.sync.items.EventChanges;
import org.obm.sync.utils.DOMUtils;
import org.obm.sync.utils.DateHelper;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Serializes calendar related items to XML
 * 
 * @author tom
 * 
 */
public class CalendarItemsWriter extends AbstractItemsWriter {

	public Document writeChanges(EventChanges cc) {
		Document doc = null;
		try {
			doc = DOMUtils.createDoc(
					"http://www.obm.org/xsd/sync/calendar-changes.xsd",
					"calendar-changes");
			Element root = doc.getDocumentElement();
			root
					.setAttribute("lastSync", DateHelper.asString(cc
							.getLastSync()));

			Element removed = DOMUtils.createElement(root, "removed");
			for (String eid : cc.getRemoved()) {
				Element e = DOMUtils.createElement(removed, "event");
				e.setAttribute("id", eid);
			}

			Element updated = DOMUtils.createElement(root, "updated");
			for (Event ev : cc.getUpdated()) {
				appendEvent(updated, ev);
			}

		} catch (Exception ex) {
			logger.error(ex.getMessage(), ex);
		}
		return doc;
	}

	public void appendEvent(Element parent, Event ev) {
		appendEvent(parent, ev, "event");
	}

	private void appendEvent(Element parent, Event ev, String eventNodeName) {
		Element e = parent;
		if (!eventNodeName.equals(parent.getNodeName())) {
			e = DOMUtils.createElement(parent, eventNodeName);
		}
		e.setAttribute("type", ev.getType().toString());
		e.setAttribute("allDay", "" + ev.isAllday());
		e.setAttribute("id", ev.getUid());
		if (ev.getTimeUpdate() != null) {
			createIfNotNull(e, "timeupdate", ""
					+ DateHelper.asString(ev.getTimeUpdate()));
		}
		createIfNotNull(e, "parentId", "" + ev.getParentId());
		createIfNotNull(e, "extId", ev.getExtId());
		createIfNotNull(e, "opacity", ev.getOpacity().toString());
		createIfNotNull(e, "title", ev.getTitle());
		createIfNotNull(e, "description", ev.getDescription());
		createIfNotNull(e, "owner", ev.getOwner());
		createIfNotNull(e, "ownerEmail", ev.getOwnerEmail());
		if (ev.getDate() != null) {
			createIfNotNull(e, "date", DateHelper.asString(ev.getDate()));
		}
		createIfNotNull(e, "duration", "" + ev.getDuration());
		createIfNotNull(e, "category", ev.getCategory());
		createIfNotNull(e, "location", ev.getLocation());
		if (ev.getAlert() != null) {
			createIfNotNull(e, "alert", "" + ev.getAlert());
		}

		createIfNotNull(e, "priority", (ev.getPriority() != null ? ""
				+ ev.getPriority() : "0"));
		createIfNotNull(e, "privacy", "" + ev.getPrivacy());
		Element atts = DOMUtils.createElement(e, "attendees");
		List<Attendee> la = ev.getAttendees();
		if (la != null) {
			for (Attendee a : ev.getAttendees()) {
				Element at = DOMUtils.createElement(atts, "attendee");
				at.setAttribute("displayName", a.getDisplayName());
				at.setAttribute("email", a.getEmail());
				at.setAttribute("state",
						(a.getState() != null ? a.getState().toString()
								: ParticipationState.NEEDSACTION.toString()));
				at.setAttribute("required", (a.getRequired() != null ? a
						.getRequired().toString() : ParticipationRole.OPT
						.toString()));
				at.setAttribute("percent", "" + a.getPercent());
			}
		}

		if (ev.getRecurrence() == null) {
			EventRecurrence er = new EventRecurrence();
			er.setKind(RecurrenceKind.none);
			ev.setRecurrence(er);
		}

		Element rec = DOMUtils.createElement(e, "recurrence");
		EventRecurrence r = ev.getRecurrence();
		rec.setAttribute("kind", r.getKind().toString());
		if (!"none".equals(r.getKind())) {
			if (r.getEnd() != null) {
				rec.setAttribute("end", DateHelper.asString(r.getEnd()));
			}
			if (r.getKind() == RecurrenceKind.weekly) {
				rec.setAttribute("days", r.getDays());
			}

			rec.setAttribute("freq", "" + r.getFrequence());
			Element exc = DOMUtils.createElement(rec, "exceptions");
			for (Date ex : r.getExceptions()) {
				DOMUtils.createElementAndText(exc, "exception", DateHelper
						.asString(ex));
			}

			Element eventExp = DOMUtils.createElement(rec, "eventExceptions");

			for (Event ex : r.getEventExceptions()) {
				appendEvent(eventExp, ex, "eventException");
			}
		}
	}

	public void appendInfo(Element root, CalendarInfo ci) {
		Element info = DOMUtils.createElement(root, "info");

		if (ci.getFirstname() != null) {
			DOMUtils.createElementAndText(info, "first", ci.getFirstname());
		}

		DOMUtils.createElementAndText(info, "last", ci.getLastname());
		DOMUtils.createElementAndText(info, "mail", ci.getMail());
		DOMUtils.createElementAndText(info, "uid", ci.getUid());
		DOMUtils.createElementAndText(info, "read", "" + ci.isRead());
		DOMUtils.createElementAndText(info, "write", "" + ci.isWrite());
	}

	public String getEventString(Event event) {
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		try {
			Document doc = DOMUtils.createDoc(
					"http://www.obm.org/xsd/sync/event.xsd", "event");
			Element root = doc.getDocumentElement();
			appendEvent(root, event);
			DOMUtils.serialise(doc, out);
		} catch (Exception e) {
			logger.error("Error writing event as string", e);
		}
		return out.toString();
	}

	public String getListEventString(List<Event> events) {
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		try {
			Document doc = DOMUtils.createDoc(
					"http://www.obm.org/xsd/sync/events.xsd", "events");
			Element root = doc.getDocumentElement();
			for (Event event : events) {
				appendEvent(root, event);
			}
			DOMUtils.serialise(doc, out);
		} catch (Exception e) {
			logger.error("Error writing event as string", e);
		}
		return out.toString();
	}

	public void appendCategory(Element root, Category c) {
		Element cat = DOMUtils.createElement(root, "cat");
		cat.setAttribute("id", "" + c.getId());
		cat.setAttribute("label", c.getLabel());
	}

	public void appendEventTimeUpdate(Element parent, EventTimeUpdate ev) {
		Element e = DOMUtils.createElement(parent, "eventTimeUpdate");
		e.setAttribute("id", ev.getUid());
		if (ev.getTimeUpdate() != null) {
			createIfNotNull(e, "timeupdate", ""
					+ DateHelper.asString(ev.getTimeUpdate()));
		}
		createIfNotNull(e, "extId", ev.getExtId());
		createIfNotNull(e, "parentId", "" + ev.getParentId());
	}

}
