/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Copyright (c) 1997-2008 Aliasource - Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  http://www.obm.org/                                              
 * 
 * ***** END LICENSE BLOCK ***** */
package fr.aliacom.obm.common;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.obm.sync.base.IStoppable;

/**
 * Map implementation removing unused values automatically after a given
 * timeout. Values in this map are all {@link IStoppable} implementations. The
 * stop method of the element is called when an item is timed out.
 * 
 * @author tom
 * 
 */
public class TimeOutMap<K, V extends IStoppable> implements Map<K, V> {

	private static Log logger = LogFactory.getLog(TimeOutMap.class);
	private Map<K, TimedItem<V>> map;
	private long timeout;

	private class Remover extends TimerTask {

		public Remover() {
		}

		public void run() {
			// remove old entries
			synchronized (map) {
				Iterator<K> it = map.keySet().iterator();
				long curTime = System.currentTimeMillis();
				while (it.hasNext()) {
					Object key = it.next();
					TimedItem<V> ti = map.get(key);
					long delay = curTime - ti.getTime();
					if (delay > timeout) {
						logger.info("Removing expired entry");
						ti.getValue().stop();
						it.remove();
					}
				}
			}
		}
	}

	/**
	 * @param timeout
	 *            unused elements timeout in milliseconds
	 */
	public TimeOutMap(long timeout) {
		map = Collections.synchronizedMap(new HashMap<K, TimedItem<V>>());
		this.timeout = timeout;
		Timer t = new Timer();
		t.scheduleAtFixedRate(new Remover(), 1000, timeout);
	}

	/**
	 * calls stop on all elements in this map, and clear it.
	 * 
	 * @see java.util.Map#clear()
	 */
	public void clear() {
		for (K key : map.keySet()) {
			IStoppable stoppable = get(key);
			stoppable.stop();
		}
		map.clear();
	}

	public boolean containsKey(Object key) {
		return map.containsKey(key);
	}

	public boolean containsValue(Object value) {
		return map.containsValue(value);
	}

	public boolean isEmpty() {
		return map.isEmpty();
	}

	public Set<K> keySet() {
		return map.keySet();
	}

	public V put(K key, V value) {
		map.put(key, new TimedItem<V>(System.currentTimeMillis(), value));
		return value;
	}

	public int size() {
		return map.size();
	}

	public V get(Object key) {
		TimedItem<V> t = map.get(key);
		if (t != null) {
			return t.getValue();
		} else {
			return null;
		}
	}

	public Set<java.util.Map.Entry<K, V>> entrySet() {
		throw new RuntimeException("not implemented");
	}

	public void putAll(Map<? extends K, ? extends V> m) {
		long cur = System.currentTimeMillis();
		for (K key : m.keySet()) {
			map.put(key, new TimedItem<V>(cur, m.get(key)));
		}
	}

	public V remove(Object key) {
		return map.remove(key).getValue();
	}

	public Collection<V> values() {
		throw new RuntimeException("not implemented");
	}

}
