/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Copyright (c) 1997-2008 Aliasource - Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  http://www.obm.org/                                              
 * 
 * ***** END LICENSE BLOCK ***** */
package fr.aliacom.obm.common;

import java.rmi.RemoteException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.obm.sync.auth.AccessToken;
import org.obm.sync.auth.AuthFault;
import org.obm.sync.server.auth.AuthentificationServiceFactory;
import org.obm.sync.server.auth.IAuthentificationService;

import fr.aliacom.obm.services.constant.ConstantService;
import fr.aliacom.obm.utils.Helper;
import fr.aliacom.obm.utils.ObmHelper;

/**
 * Helpers for working with userobm & domain tables Authentification method is
 * also here
 * 
 * @author tom
 */
public class UserManagement {

	private static final UserManagement instance = new UserManagement();

	/**
	 * @return the server instance
	 */
	public static UserManagement getInstance() {
		return instance;
	}

	private TimeOutMap<String, AccessToken> sessions;
	private Map<String, ObmDomain> domainCache;

	private static final Log logger = LogFactory.getLog(UserManagement.class);

	private UserManagement() {
		sessions = new TimeOutMap<String, AccessToken>(1000 * 60 * 5);
		domainCache = Collections
				.synchronizedMap(new HashMap<String, ObmDomain>());
	}

	private String newSessionId() {
		return UUID.randomUUID().toString();
	}

	/**
	 * 
	 * @param login
	 * @param domain
	 * @return null if the credential are not valid
	 */
	public AccessToken login(String login, String password, String origin) {
		if (logger.isDebugEnabled()) {
			logger.debug("Login trial for login: " + login + " pass: "
					+ password);
		}

		String[] splited = login.split("@", 2);
		String userLogin = splited[0];
		String userDomain = null;
		AccessToken token = null;

		if (splited.length == 2) {
			userDomain = splited[1];
		}

		if (userDomain == null
				&& !ConstantService.getInstance().getBooleanValue(
						"singleNameSpace")) {
			logger
					.warn("rejecting login as single name space is not enabled and @domain not given in login ("
							+ userLogin + ").");
			return null;
		}

		IAuthentificationService authService = new AuthentificationServiceFactory()
				.get();

		if (userDomain == null) {
			userDomain = authService.getObmDomain(userLogin);
		}
		ObmDomain obmDomain = findDomainByName(userDomain);
		if (obmDomain == null) {
			logger.warn("cannot figure out domain for the domain_name "
					+ userDomain);
			return null;
		}

		boolean valid = authService.doAuth(userLogin, obmDomain, password);
		if (valid) {
			ObmUser databaseUser = findUserByLogin(userLogin, obmDomain);
			token = new AccessToken(databaseUser.getUid(), obmDomain.getId(),
					origin);
			token.setUser(userLogin);
			token.setDomain(userDomain);
			token.setSessionId(newSessionId());
			token.setEmail(Helper.constructEmailFromList(databaseUser
					.getEmail(), obmDomain.getName()));
			sessions.put(token.getSessionId(), token);
			logger.info(login + " logged in from " + token.getOrigin()
					+ ". auth type: " + authService.getType() + " (mail: "
					+ token.getEmail() + ")");
			loadUserProperties(token);
			// loadCalendarRights(token, databaseUser);
			return token;
		} else {
			logger.info("access refused to login: '" + userLogin
					+ "' domain: '" + userDomain + "' with password '"
					+ password + "' auth type: " + authService.getType());
			return null;
		}

	}
	
	private void loadUserProperties(AccessToken token) {
		String q = "SELECT serviceproperty_service, serviceproperty_property, serviceproperty_value "
				+ "FROM ServiceProperty "
				+ "INNER JOIN UserEntity ON serviceproperty_entity_id=userentity_entity_id AND userentity_user_id="
				+ token.getObmId();
		Statement ps = null;
		ResultSet rs = null;
		Connection con = null;
		try {
			con = ObmHelper.getConnection();
			ps = con.createStatement();
			rs = ps.executeQuery(q);
			while (rs.next()) {
				String k = rs.getString(1) + "/" + rs.getString(2);
				String v = rs.getString(3);
				token.addServiceProperty(k, v);
				logger.info("found property for " + token.getUser() + "@"
						+ token.getDomain() + ": " + k + " => " + v);
			}
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		} finally {
			ObmHelper.cleanup(con, ps, rs);
		}
	}

	/**
	 * Checks the validity of the given session id
	 * 
	 * @param login
	 * @param sessionId
	 * @throws RemoteException
	 */
	public void checkToken(AccessToken at) throws AuthFault {
		if (logger.isDebugEnabled()) {
			logger.debug("checkToken sessionId:" + at.getSessionId());
		}

		AccessToken u = sessions.get(at.getSessionId());

		if (u == null) {
			throw new AuthFault("Invalid access token");
		}
		at.setDomain(u.getDomain());
		at.setUser(u.getUser());
		at.setDomainId(u.getDomainId());
		at.setObmId(u.getObmId());
		at.setOrigin(u.getOrigin());
		at.setEmail(u.getEmail());
		at.setIsoCodeToNameCache(u.getIsoCodeToNameCache());
		at.setServiceProperties(u.getServiceProperties());
		at.setCalendarRights(u.getCalendarRights());

		/* re-arm the session timer */
		sessions.put(at.getSessionId(), u);
	}

	public ObmDomain findDomainByName(String domainName) {
		if (domainCache.containsKey(domainName)) {
			return domainCache.get(domainName);
		}

		Connection con = null;
		PreparedStatement ps = null;
		ResultSet rs = null;

		ObmDomain ret = null;
		String uq = "SELECT domain_id FROM Domain WHERE domain_name=?";
		try {
			con = ObmHelper.getConnection();
			ps = con.prepareStatement(uq);
			ps.setString(1, domainName);
			rs = ps.executeQuery();
			if (rs.next()) {
				ret = new ObmDomain();
				ret.setId(rs.getInt(1));
				ret.setName(domainName);
			}
		} catch (SQLException e) {
			logger.error(e.getMessage(), e);
		} finally {
			ObmHelper.cleanup(con, ps, rs);
		}
		if (ret != null) {
			synchronized (domainCache) {
				domainCache.put(domainName, ret);
			}
		}
		return ret;
	}

	public Integer userIdFromEmailQuery(Connection con, String mail)
			throws SQLException {
		String[] parts = mail.split("@");
		String left = parts[0];
		String right = "-";
		if (parts.length > 1) {
			right = parts[1];
		}

		Integer ret = null;

		Statement st = null;
		ResultSet rs = null;
		try {
			st = con.createStatement();
			// by Mehdi
			rs = st.executeQuery("SELECT userobm_id FROM UserObm "
					+ "INNER JOIN Domain ON userobm_domain_id = domain_id "
					+ "WHERE " + "( " + "(userobm_email = '"
					+ left
					+ "' OR userobm_email like '"
					+ left
					+ "\r\n%' OR userobm_email like '%\r\n"
					+ left
					+ "\r\n%' OR userobm_email like '%\r\n"
					+ left
					+ "') "
					+ "AND "
					+ "(domain_name = '"
					+ right
					+ "' OR domain_alias = '"
					+ right
					+ "' OR domain_alias like '"
					+ right
					+ "\r\n%' OR domain_alias like '%\r\n"
					+ right
					+ "\r\n%' OR domain_alias like '%\r\n"
					+ right
					+ "') "
					+ ") OR (userobm_email = '"
					+ mail
					+ "' OR userobm_email like '"
					+ mail
					+ "\r\n%' OR userobm_email like '%\r\n"
					+ mail
					+ "\r\n%' OR userobm_email like '%\r\n" + mail + "') ");
			if (rs.next()) {
				ret = rs.getInt(1);
			}

		} finally {
			ObmHelper.cleanup(null, st, rs);
		}
		return ret;
	}

	public Integer contactEntityFromEmailQuery(Connection con, String mail)
			throws SQLException {
		Integer ret = null;
		String q = "SELECT email_entity_id FROM Email "
				// prevents linking to an orphan email row
				+ "INNER JOIN ContactEntity ON email_entity_id=contactentity_entity_id "
				+ "WHERE email_address=?";
		PreparedStatement st = null;
		ResultSet rs = null;
		try {
			st = con.prepareStatement(q);
			st.setString(1, mail);
			rs = st.executeQuery();
			if (rs.next()) {
				ret = rs.getInt(1);
			}
		} catch (SQLException se) {
			logger.error(se.getMessage(), se);
		} finally {
			ObmHelper.cleanup(null, st, rs);
		}
		return ret;
	}

	public Integer userEntityFromEmailQuery(Connection con, String mail)
			throws SQLException {
		String[] parts = mail.split("@");
		String left = parts[0];
		String right = "-";
		if (parts.length > 1) {
			right = parts[1];
		}

		Integer ret = null;

		Statement st = null;
		ResultSet rs = null;
		try {
			st = con.createStatement();
			// by Mehdi
			rs = st
					.executeQuery("SELECT userentity_entity_id FROM UserObm INNER JOIN UserEntity ON userobm_id=userentity_user_id "
							+ "INNER JOIN Domain ON userobm_domain_id = domain_id "
							+ "WHERE " + "( " + "(userobm_email = '"
							+ left
							+ "' OR userobm_email like '"
							+ left
							+ "\r\n%' OR userobm_email like '%\r\n"
							+ left
							+ "\r\n%' OR userobm_email like '%\r\n"
							+ left
							+ "') "
							+ "AND "
							+ "(domain_name = '"
							+ right
							+ "' OR domain_alias = '"
							+ right
							+ "' OR domain_alias like '"
							+ right
							+ "\r\n%' OR domain_alias like '%\r\n"
							+ right
							+ "\r\n%' OR domain_alias like '%\r\n"
							+ right
							+ "') "
							+ ") OR (userobm_email = '"
							+ mail
							+ "' OR userobm_email like '"
							+ mail
							+ "\r\n%' OR userobm_email like '%\r\n"
							+ mail
							+ "\r\n%' OR userobm_email like '%\r\n"
							+ mail
							+ "') ");
			if (rs.next()) {
				ret = rs.getInt(1);
			}

		} finally {
			ObmHelper.cleanup(null, st, rs);
		}
		return ret;
	}

	public ObmUser findUser(String email, String displayName, ObmDomain domain) {
		Connection con = null;
		Integer id = null;
		try {
			con = ObmHelper.getConnection();
			id = userIdFromEmailQuery(con, email);
		} catch (SQLException se) {
			logger.error(se.getMessage(), se);
		} finally {
			ObmHelper.cleanup(con, null, null);
		}
		if (id != null && id > 0) {
			return findUserById(id);
		} else {
			return null;
		}
	}

	/**
	 * does not return archived users
	 * 
	 * @param login
	 * @param domain
	 * @return
	 */
	public ObmUser findUserByLogin(String login, ObmDomain domain) {
		Connection con = null;
		PreparedStatement ps = null;
		ResultSet rs = null;

		ObmUser ret = null;
		String uq = "SELECT userobm_id, userobm_email, userobm_firstname, userobm_lastname FROM UserObm "
				+ "WHERE userobm_domain_id=? AND userobm_login=? AND userobm_archive != '1'";
		try {
			con = ObmHelper.getConnection();
			ps = con.prepareStatement(uq);
			ps.setInt(1, domain.getId());
			ps.setString(2, login);
			rs = ps.executeQuery();
			if (rs.next()) {
				ret = new ObmUser();
				ret.setUid(rs.getInt(1));
				ret.setLogin(login);
				ret.setDomain(domain);
				ret.setEmail(rs.getString(2));
				ret.setFirstName(rs.getString(3));
				ret.setLastName(rs.getString(4));
			}
		} catch (SQLException e) {
			logger.error(e.getMessage(), e);
		} finally {
			ObmHelper.cleanup(con, ps, rs);
		}
		return ret;
	}

	public ObmUser findUserById(int id) {
		Connection con = null;
		PreparedStatement ps = null;
		ResultSet rs = null;

		ObmUser ret = null;
		String uq = "SELECT userobm_id, userobm_email, userobm_login FROM UserObm "
				+ "WHERE userobm_id=?";
		try {
			con = ObmHelper.getConnection();
			ps = con.prepareStatement(uq);
			ps.setInt(1, id);
			rs = ps.executeQuery();
			if (rs.next()) {
				ret = new ObmUser();
				ret.setUid(rs.getInt(1));
				ret.setEmail(rs.getString(2));
				ret.setLogin(rs.getString(3));
			}
		} catch (SQLException e) {
			logger.error(e.getMessage(), e);
		} finally {
			ObmHelper.cleanup(con, ps, rs);
		}
		return ret;
	}

	/**
	 * Returns an ObmUser if the calendar@domainName has an email and is not
	 * archived
	 * 
	 * @param calendar
	 * @param domainName
	 * @return
	 * @throws FindException
	 */
	public ObmUser checkCalendarUser(String calendar, String domainName)
			throws FindException {
		ObmDomain domain = findDomainByName(domainName);
		ObmUser user = findUserByLogin(calendar, domain);
		if (user == null || user.getEmail() == null
				|| user.getEmail().equals("")) {
			logger.info("user :" + calendar
					+ " not found, archived or have no email");
			throw new FindException("Calendar not exist or not valid");
		}
		return user;
	}

	public void logout(String sessionId) {
		logger.info("logout for " + sessionId + "...");
		sessions.remove(sessionId);
	}

}