/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Copyright (c) 1997-2008 Aliasource - Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  http://www.obm.org/                                              
 * 
 * ***** END LICENSE BLOCK ***** */
package fr.aliacom.obm.common.calendar;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.Date;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.obm.sync.auth.AccessToken;
import org.obm.sync.auth.AuthFault;
import org.obm.sync.auth.ServerFault;
import org.obm.sync.base.Category;
import org.obm.sync.base.KeyList;
import org.obm.sync.calendar.CalendarInfo;
import org.obm.sync.calendar.Event;
import org.obm.sync.calendar.EventTimeUpdate;
import org.obm.sync.calendar.EventType;
import org.obm.sync.items.EventChanges;
import org.obm.sync.services.ICalendar;

import fr.aliacom.obm.common.FindException;
import fr.aliacom.obm.common.ObmDomain;
import fr.aliacom.obm.common.ObmUser;
import fr.aliacom.obm.common.UserManagement;
import fr.aliacom.obm.utils.Helper;
import fr.aliacom.obm.utils.Ical4jHelper;

public class CalendarBindingImpl implements ICalendar {

	private static final Log logger = LogFactory
			.getLog(CalendarBindingImpl.class);

	private UserManagement um;

	private EventType type;

	public CalendarBindingImpl(EventType type) {
		this.um = UserManagement.getInstance();
		this.type = type;
	}

	public CalendarInfo[] listCalendars(AccessToken token) throws ServerFault,
			AuthFault {
		um.checkToken(token);

		try {
			List<CalendarInfo> l = getRights(token);

			CalendarInfo[] ret = (CalendarInfo[]) l.toArray(new CalendarInfo[l
					.size()]);
			logger.info("Returning " + ret.length + " calendar infos.");
			return ret;
		} catch (Throwable e) {
			logger.error(e.getMessage(), e);
			throw new ServerFault(e.getMessage());
		}
	}

	private List<CalendarInfo> listCalendarsImpl(AccessToken token) throws ServerFault,
			AuthFault {
		um.checkToken(token);

		ObmUser u = null;
		try {
			u = um.checkCalendarUser(token.getUser(), token.getDomain());
		} catch (FindException e1) {
			throw new ServerFault(e1.getMessage());
		}

		try {
			return CalendarService.getInstance().listCalendars(u);
		} catch (Throwable e) {
			logger.error(e.getMessage(), e);
			throw new ServerFault(e.getMessage());
		}
	}

	private List<CalendarInfo> getRights(AccessToken t) throws ServerFault, AuthFault {
		List<CalendarInfo> rights = t.getCalendarRights();
		if (rights == null) {
			rights = listCalendarsImpl(t);
			t.setCalendarRights(rights);
		}
		return rights;
	}

	public Event removeEvent(AccessToken token, String calendar, String uid)
			throws AuthFault, ServerFault {
		um.checkToken(token);
		try {
			ObmDomain domain = UserManagement.getInstance().findDomainByName(
					token.getDomain());
			ObmUser u = um.findUserByLogin(token.getUser(), domain);

			List<CalendarInfo> rights = getRights(token);

			// only remove if logged user is owner
			// or as write right on owner
			Event ev = CalendarService.getInstance().findEvent(token, uid,
					calendar);

			if (ev != null
					&& ev.getOwner() != null
					&& !(ev.getOwner().equals(u.getLogin()) || Helper
							.hasWriteRightOn(rights, ev.getOwner()))) {
				logger.info("remove not allowed of " + ev.getTitle());
				// mark as updated : will be send to client on second sync
				ev = CalendarService.getInstance().markEventAsUpdated(ev);
				return ev;
			}

			ev = CalendarService.getInstance()
					.removeEvent(token, calendar, uid);
			logger.info("[" + token.getUser() + "] Calendar : event[" + uid
					+ "] removed");
			return ev;
		} catch (Throwable e) {
			logger.error(e.getMessage(), e);
			throw new ServerFault(e.getMessage());
		}
	}

	/**
	 * @throws AuthFault
	 * @throws ServerFault
	 */
	public Event modifyEvent(AccessToken token, String calendar, Event event,
			boolean updateAttendees) throws AuthFault, ServerFault {
		um.checkToken(token);

		if (event == null) {
			logger.warn("Modify on NULL event: doing nothing");
			return null;
		}

		try {
			ObmDomain domain = um.findDomainByName(token.getDomain());
			ObmUser u = um.findUserByLogin(token.getUser(), domain);

			boolean onlyUpdateMyself = false;
			List<CalendarInfo> rights = getRights(token);

			// only modify if owner or write right on owner
			Event ev = CalendarService.getInstance().findEvent(token,
					event.getUid(), calendar);

			if (rights != null
					&& ev != null
					&& ev.getOwner() != null
					&& !(ev.getOwner().equals(u.getLogin()) || Helper
							.hasWriteRightOn(rights, ev.getOwner()))) {
				logger.info("[" + token.getUser() + "] Calendar : "
						+ token.getUser() + " cannot modify event["
						+ ev.getTitle() + "] because not owner"
						+ " or no write right on owner " + ev.getOwner());
				onlyUpdateMyself = true;
			}

			ev = CalendarService.getInstance().modifyEvent(token, event,
					onlyUpdateMyself, updateAttendees);

			if (ev != null && !onlyUpdateMyself) {
				logger.info("[" + token.getUser() + "] Calendar : event["
						+ ev.getTitle() + "] modified");
			}
			return ev;
		} catch (Throwable e) {
			logger.error(e.getMessage(), e);
			throw new ServerFault(e.getMessage());
		}

	}

	/**
	 * @throws AuthFault
	 * @throws ServerFault
	 * @see fr.aliacom.obm.ecalendar.Calendar_PortType#createEvent(fr.aliacom.obm.ecalendar.AccessToken,
	 *      java.lang.String, fr.aliacom.obm.ecalendar.Event)
	 */
	public String createEvent(AccessToken token, String calendar, Event event)
			throws AuthFault, ServerFault {
		um.checkToken(token);

		try {
			if (event == null) {
				logger.warn("[" + token.getUser()
						+ "] syncing NULL event, returning fake id 0");
				return ("0");
			}

			ObmDomain domain = um.findDomainByName(token.getDomain());

			List<String> duplicates = CalendarService.getInstance()
					.findEventTwinKeys(calendar, event, domain);
			if (duplicates.size() > 0) {
				logger.info("[" + token.getUser() + "] Calendar : "
						+ duplicates.size() + " duplicate found for event ["
						+ event.getTitle() + "," + event.getDate() + ","
						+ event.getDuration() + "," + event.isAllday() + "]");
				return duplicates.get(0);
			}

			List<CalendarInfo> rights = getRights(token);

			// create only if write right
			if (!Helper.hasWriteRightOn(rights, calendar)) {
				Helper.resetAttendeesStatus(event);
				/*
				 * logger.info("[" + token.getUser() + "] Calendar : " +
				 * token.getUser() + " cannot create event on " + calendar + "
				 * calendar : no write right");
				 */
			}

			Event ev = CalendarService.getInstance().createEvent(token, event);

			logger.info("[" + token.getUser() + "] Calendar : event["
					+ ev.getTitle() + "] created");
			return ("" + ev.getDatabaseId());
		} catch (Throwable e) {
			logger.error(e.getMessage(), e);
			throw new ServerFault(e.getMessage());
		}
	}

	public EventChanges getSync(AccessToken token, String calendar,
			Date lastSync) throws AuthFault, ServerFault {
		UserManagement.getInstance().checkToken(token);

		ObmUser calendarUser = null;
		try {
			calendarUser = UserManagement.getInstance().checkCalendarUser(
					calendar, token.getDomain());
		} catch (FindException e1) {
			throw new ServerFault(e1.getMessage());
		}

		try {

			logger.info("[" + token.getUser() + "] Calendar : getSync("
					+ calendar + ", " + lastSync + ")");

			EventChanges sync = CalendarService.getInstance().getSync(token,
					calendarUser, lastSync, type);
			logger.info("getSync(" + calendar + ") => "
					+ sync.getUpdated().length + " upd, "
					+ sync.getRemoved().length + " rmed.");
			return sync;
		} catch (Throwable e) {
			logger.error(e.getMessage(), e);
			throw new ServerFault(e.getMessage());
		}
	}

	// methods for Funambol sync

	public Event getEventFromId(AccessToken token, String calendar, String id)
			throws AuthFault, ServerFault {
		um.checkToken(token);
		try {
			return CalendarService.getInstance().findEvent(token, id, calendar);
		} catch (Throwable e) {
			logger.error(e.getMessage(), e);
			throw new ServerFault(e.getMessage());
		}
	}

	public KeyList getEventTwinKeys(AccessToken token, String calendar,
			Event event) throws AuthFault, ServerFault {
		UserManagement.getInstance().checkToken(token);
		try {
			ObmDomain domain = um.findDomainByName(token.getDomain());
			List<String> keys = CalendarService.getInstance()
					.findEventTwinKeys(calendar, event, domain);
			logger.info("found " + keys.size() + " twinkeys ");
			return new KeyList(keys, null);
		} catch (Throwable e) {
			logger.error(e.getMessage(), e);
			throw new ServerFault(e.getMessage());
		}
	}

	public KeyList getRefusedKeys(AccessToken token, String calendar, Date since)
			throws AuthFault, ServerFault {
		um.checkToken(token);

		try {
			ObmDomain domain = um.findDomainByName(token.getDomain());
			ObmUser calendarUser = um.findUserByLogin(calendar, domain);
			List<String> keys = CalendarService.getInstance()
					.findRefusedEventsKeys(calendarUser, since);

			return new KeyList(keys, null);
		} catch (Throwable e) {
			logger.error(e.getMessage(), e);
			throw new ServerFault(e.getMessage());
		}
	}

	public List<Category> listCategories(AccessToken at) throws ServerFault,
			AuthFault {
		um.checkToken(at);

		try {
			List<Category> c = CategoryService.getInstance().getCategories(at);
			return c;
		} catch (Throwable e) {
			logger.error(e.getMessage(), e);
			throw new ServerFault(e.getMessage());
		}
	}

	public String getUserEmail(AccessToken token) throws AuthFault, ServerFault {
		um.checkToken(token);

		try {
			ObmUser obmuser = um.findUserById(token.getObmId());
			if (obmuser != null) {
				return Helper.constructEmailFromList(obmuser.getEmail(), token
						.getDomain());
			} else {
				return "";
			}

		} catch (Throwable e) {
			logger.error(e.getMessage(), e);
			throw new ServerFault(e.getMessage());
		}
	}

	public Event getEventFromExtId(AccessToken token, String calendar,
			String extId) throws AuthFault, ServerFault {
		um.checkToken(token);
		try {
			return CalendarService.getInstance().findEventByExtId(token, extId,
					calendar);
		} catch (Throwable e) {
			logger.error(e.getMessage(), e);
			throw new ServerFault(e.getMessage());
		}
	}

	@Override
	public List<Event> getListEventsFromIntervalDate(AccessToken token,
			String calendar, Date start, Date end) throws AuthFault,
			ServerFault {
		um.checkToken(token);
		ObmUser calendarUser = null;
		try {
			calendarUser = UserManagement.getInstance().checkCalendarUser(
					calendar, token.getDomain());
		} catch (FindException e1) {
			throw new ServerFault(e1.getMessage());
		}
		try {
			return CalendarService.getInstance()
					.findListEventsFromIntervalDate(token, calendarUser, start,
							end, type);
		} catch (Throwable e) {
			logger.error(e.getMessage(), e);
			throw new ServerFault(e.getMessage());
		}
	}

	@Override
	public List<Event> getAllEvents(AccessToken token, String calendar,
			EventType eventType) throws AuthFault, ServerFault {
		um.checkToken(token);
		ObmUser calendarUser = null;
		try {
			calendarUser = UserManagement.getInstance().checkCalendarUser(
					calendar, token.getDomain());
			return CalendarService.getInstance().findAllEvents(token,
					calendarUser, eventType);
		} catch (Throwable e) {
			logger.error(e.getMessage(), e);
			throw new ServerFault(e.getMessage());
		}
	}

	@Override
	public List<EventTimeUpdate> getAllEventTimeUpdate(AccessToken token,
			String calendar, EventType eventType) throws AuthFault, ServerFault {

		um.checkToken(token);

		ObmUser calendarUser = null;
		try {
			calendarUser = UserManagement.getInstance().checkCalendarUser(
					calendar, token.getDomain());
			return CalendarService.getInstance().findAllEventTimeUpdate(token,
					calendarUser, eventType);
		} catch (Throwable e) {
			logger.error(e.getMessage(), e);
			throw new ServerFault(e.getMessage());
		}
	}

	public String parseEvent(AccessToken token, Event event)
			throws ServerFault, AuthFault {
		um.checkToken(token);
		return Ical4jHelper.parseEvent(event, token);
	}

	public String parseEvents(AccessToken token, List<Event> events)
			throws ServerFault, AuthFault {
		um.checkToken(token);
		return Ical4jHelper.parseEvents(events);
	}

	public List<Event> parseICS(AccessToken token, String ics)
			throws Exception, ServerFault {
		um.checkToken(token);
		InputStream is = new ByteArrayInputStream(ics.getBytes());
		return Ical4jHelper.parseICS(is, token);
	}
}