/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Copyright (c) 1997-2008 Aliasource - Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  http://www.obm.org/                                              
 * 
 * ***** END LICENSE BLOCK ***** */
package fr.aliacom.obm.ldap;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.NamingException;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.naming.directory.SearchResult;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.obm.sync.server.auth.IAuthentificationService;

import fr.aliacom.obm.common.ObmDomain;
import fr.aliacom.obm.common.contact.UserHome;

/**
 * Effectue l'authentification sur un annaire ldap.
 * 
 * @author tom
 * 
 */
public class LDAPAuthService implements IAuthentificationService {

	public static final String LDAP_CONF = "/etc/obm/ldap-auth.xml";

	private static LDAPAuthService ldapAuth;

	private LDAPAuthConfig authConfig;
	private static Log logger = LogFactory.getLog(LDAPAuthService.class);
	private LDAPDirectory directory;

	static {
		try {
			ldapAuth = new LDAPAuthService();
		} catch (FileNotFoundException e) {
			logger.error("error reading LDAP auth configuration from "
					+ LDAP_CONF, e);
		}
	}

	private LDAPAuthService() throws FileNotFoundException {
		if (new File(LDAP_CONF).exists()) {
			this.authConfig = new LDAPAuthConfig(new FileInputStream(LDAP_CONF));
		} else {
			this.authConfig = new LDAPAuthConfig();
		}
		this.directory = authConfig.getDirectory();
	}

	public static LDAPAuthService get() {
		return ldapAuth;
	}

	public String getObmDomain(String login) {
		String ret = directory.getObmDomain();
		if (ret == null) {
			ret = UserHome.getInstance().getUserDomain(login);
		}
		return ret;
	}

	public boolean doAuth(String login, ObmDomain domain,
			String clearTextPassword) {
		if (directory.getRootDN() != null) {
			return doLookupAuth(login, domain.getName(), clearTextPassword);
		} else {
			return doBindAuth(login, domain.getName(), clearTextPassword);
		}
	}

	private boolean doLookupAuth(String login, String domain,
			String clearTextPassword) {
		if (logger.isDebugEnabled()) {
			logger
					.debug("doLookupAuth l/p: " + login + "/"
							+ clearTextPassword);
		}
		LDAPUtils utils = new LDAPUtils(directory.getUri(), directory
				.getRootDN(), directory.getRootPW(), directory.getBaseDN());
		return utils.checkPassword(directory.getMatchField(), login,
				clearTextPassword);
	}

	private boolean doBindAuth(String login, String domain,
			String clearTextPassword) {
		if (logger.isDebugEnabled()) {
			logger.debug("doBindAuth l/p: " + login + "/" + clearTextPassword);
		}

		Hashtable<String, String> env = new Hashtable<String, String>();
		env.put(Context.INITIAL_CONTEXT_FACTORY,
				"com.sun.jndi.ldap.LdapCtxFactory");
		env.put(Context.PROVIDER_URL, directory.getUri());
		env.put(Context.SECURITY_AUTHENTICATION, "none");

		LDAPUtils utils = new LDAPUtils(env, directory.getBaseDN());
		boolean ret = false;
		DirContext lookup = null;
		DirContext bind = null;
		String dn = null;
		try {
			lookup = utils.getConnection();
			SearchResult sr = utils.findResultByFilter(directory
					.getUserPattern().replace("%u", login)
					.replace("%d", domain), lookup);
			if (sr != null) {
				dn = (String) sr.getName() + "," + directory.getBaseDN();
				logger.info("dn lookup: " + login + "@" + domain + " => " + dn);
			}

		} catch (Exception e) {
			logger.error(e.getMessage() + " (with password "
					+ clearTextPassword + " and dn: " + dn + ")", e);
		} finally {
			if (lookup != null) {
				try {
					lookup.close();
				} catch (NamingException e) {
				}
			}
		}
		if (dn != null) {
			try {
				Hashtable<String, String> bindenv = new Hashtable<String, String>();
				bindenv.put(Context.INITIAL_CONTEXT_FACTORY,
						"com.sun.jndi.ldap.LdapCtxFactory");
				bindenv.put(Context.PROVIDER_URL, directory.getUri());
				bindenv.put(Context.SECURITY_AUTHENTICATION, "simple");
				bindenv.put(Context.SECURITY_PRINCIPAL, dn);
				bindenv.put(Context.SECURITY_CREDENTIALS, clearTextPassword);
				bind = new InitialDirContext(bindenv);
				ret = true;
			} catch (Exception e) {
				logger.error(e.getMessage() + " (with password "
						+ clearTextPassword + " and dn: " + dn + ")", e);
			} finally {
				if (bind != null) {
					try {
						bind.close();
					} catch (NamingException e) {
					}
				}
			}
		}
		return ret;
	}

	public String getType() {
		return "LDAP";
	}

}
